//###########################################################
// File: dir.c
//
// Directory functions
//
// For FAT12, FAT16
// Only for first Partition
// Only for drives with 512 bytes per sector (the most)
//
// Benutzung auf eigene Gefahr !
//
// Use at your own risk !
//
//#########################################################################
// Last change: 29.07.2005
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.3
//#########################################################################
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "dos.h"

//############################################################
// search for a filename or directoryname in current directory
unsigned char FindName(char *name)
//############################################################
{
 U8 i;
 unsigned char result;

 result=NO_MATCH;

 for(i=0; i<RootDirSectors; i++)
  {
   result=ScanOneDirectorySector(FirstRootSector+i, name);
   if(result!=NO_MATCH) break; //break sector loop
  }

 return result;
}

//###########################################################
// find a filename
//
// Following global variables will be set if filename
// was found:
//
// FileFirstCluster  First cluster of the file
// FileSize
//
unsigned char ScanOneDirectorySector(unsigned long sector, char *name)
//###########################################################
{
 U16 count;
 unsigned long tmp;
 struct DirEntry *di;
 char nam[8];
 unsigned char i,match;

 MakeFileName(name,nam);
 
// by=ReadSector(sector,dirbuf); //read one directory sector.
 count=0;
 match=NO_MATCH;

#ifdef MMC_CARD_SPI
   MMC_CS_OFF();
   MMCStartReadSector(sector);
#endif

#ifdef COMPACTFLASH_CARD
   CFStartReadSector(sector);
   CF_CS_OFF();
#endif

 do
  {
   for(i=0; i<32; i++)
    {
#ifdef MMC_CARD_SPI
     SPI_WRITE(0xFF);
     dirbuf[i]=SPDR;
#endif

#ifdef COMPACTFLASH_CARD
     CF_RD_OFF();
     NOP; //Schon bei 8MHz mu man hier etwas warten
     NOP; //Es knnte evtl. ntig sein hier noch mehr
        //nop einzufgen, um mglichst viele CF zu lesen
        //Bei mir reichten bisher 2 nop
        //Sicherheitshalber noch einen mehr
     NOP;

#ifdef F_16 //16MHz processor clock
     NOP;
#endif
   
     dirbuf[i]=CF_READ_DATA();
     CF_RD_ON();
#endif
    }

   di=(struct DirEntry *)dirbuf;
   
   if((unsigned char)di->DIR_Name[0]!=0xE5 && di->DIR_Name[0]>0 && match!=FULL_MATCH)
    {
     di->DIR_Attr&=0x3F;            //smash upper two bits

     if(di->DIR_Attr==ATTR_LONG_NAME)
      {
      }
     else
      {
       if(strncmp(nam,di->DIR_Name,8)==0) match=FULL_MATCH;
                
       if(di->DIR_Attr & ATTR_VOLUME_ID)
        {
        }
       else
        {
         if(di->DIR_Attr & ATTR_DIRECTORY) //this is a directory
          {
          }//if(di->DIR_Attr & ATTR_DIRECTORY)
         else //is not a directory. this is a file
          {
           tmp=di->DIR_FstClusHI; //Highword of first cluster number
           tmp<<=16;
           tmp+=di->DIR_FstClusLO; //Lowword of first cluster number

           if(match==FULL_MATCH)
            {
             FileFirstCluster=tmp;
             FileSize=di->DIR_FileSize;
            } 

          }//if(di->DIR_Attr & ATTR_DIRECTORY)
        }
      }
         
    }//if(di->DIR_Name[0]!=0xE5 && di->DIR_Name[0]>0)
        
   count+=32;
  }while(count<BYTE_PER_SEC);

#ifdef MMC_CARD_SPI
 SPI_WRITE(0xFF); //CRC
 SPI_WRITE(0xFF);
 MMC_CS_ON();
#endif

#ifdef COMPACTFLASH_CARD
 CF_CS_ON();
#endif

 return match;
}


//###########################################################
void MakeFileName(char *inname, char *outname)
//###########################################################
{
 unsigned char by,i;
 char *p;

 p=outname;
 for(i=0; i<8; i++) *p++=' '; //fill filename buffer with spaces

 p=outname;

 i=0; //get filename
 do
  {
   by=inname[i];
   if(by!=0) *p++=by;
   i++;
  }while(i<8 && by!=0);

// for(i=0; i<8; i++) putchar(outname[i]);
}


