//#########################################################################
// File: fat.h
//
// Benutzt nur die erste Partition
// Nur fr Laufwerke mit 512 Bytes pro Sektor
//
// Nach einem White Paper von MS
// FAT: General Overview of On-Disk Format
// Version 1.03, December 6, 2000
//
// Benutzung auf eigene Gefahr !
//
// Use at your own risk !
//
//#########################################################################
// Last change: 12.06.2004
//#########################################################################
// Compiler: AVR-GCC 3.2
//#########################################################################

#ifndef __FAT_H
#define __FAT_H

//file operations
#define END_DIR		0
#define NO_MATCH	1
#define MATCH_NAME	2
#define MATCH_EXT	3
#define FULL_MATCH	MATCH_NAME + MATCH_EXT

#define PART1_TABLE_OFFSET 0x01BE //offset to first partitiontable in sector 0

//Using structures needs less memory than indexing in arrays like inbuff[]

//partitiontable structure
//most of it is not used in this program
//bootsector offset is the only thing we need
//because C/H/S values are not used. LBA !
struct PartInfo {
                 unsigned char status;      //Partition status, 0x80 = Active, 0x00 = inactive
                 unsigned char firsthead;   //First head used by partition
                 U16  firstseccyl; //First sector and cylinder used by partition
                 unsigned char type;        //Partition type
                 unsigned char lasthead;    //Last head used by partition
                 U16  lastseccyl;  //Last sector and cylinder used by partition
                 unsigned long bootoffset;  //Location of boot sector. !!!!!!!!!!!
                 unsigned long secofpart;   //Number of sectors for partition
};

//first sector of disc is the master boot record
//it contains four partitiontables
//only the first partition is used in this program
struct MBR {
            unsigned char dummy[PART1_TABLE_OFFSET]; //we don't need all these bytes
            struct PartInfo part1;
            struct PartInfo part2;
            struct PartInfo part3;
            struct PartInfo part4;
//all bytes below are not necessary
};

//part of FAT12/16 bootsector different to FAT32
struct RemBoot //FAT12/16 defs beginning at offset 36
 {
	unsigned char  BS_DrvNum;
	unsigned char  BS_Reserved1;
	unsigned char  BS_BootSig;
	unsigned char  BS_VolID[4];
	char           BS_VolLab[11];
	char           BS_FilSysType[8];
	unsigned char  remaining_part[450];
 };

//part of FAT32 bootsector different to FAT12/16
struct RemBoot32 //FAT32 defs beginning at offset 36
  {
	unsigned long  BPB_FATSz32; //4 bytes
	U16   BPB_ExtFlags; //2 bytes
	U16   BPB_FSVer; //2 bytes
	unsigned long  BPB_RootClus; //4 bytes
	U16   BPB_FSInfo; //2 bytes
	U16   BPB_BkBootSec; //2 bytes
	unsigned char  BPB_Reserved[12];
	unsigned char  BS_DrvNum;
	unsigned char  BS_Reserved1;
	unsigned char  BS_BootSig;
	unsigned long  BS_VolID; //4 bytes
	char           BS_VolLab[11];
	char           BS_FilSysType[8];
	unsigned char  remaining_part[422];
}; 

union endboot 
{
       struct RemBoot   rm;
       struct RemBoot32 rm32;
};

struct BootSec 
{
//	unsigned char  BS_jmpBoot[3];
//	char           BS_OEMName[8];
//	U16   BPB_BytesPerSec; //2 bytes
	unsigned char  BPB_SecPerClus;
	U16   BPB_RsvdSecCnt; //2 bytes
	unsigned char  BPB_NumFATs;
	U16   BPB_RootEntCnt; //2 bytes
	U16   BPB_TotSec16; //2 bytes
	unsigned char  BPB_Media;
	U16   BPB_FATSz16; //2 bytes
	U16   BPB_SecPerTrk; //2 bytes
	U16   BPB_NumHeads; //2 bytes
	unsigned long  BPB_HiddSec; //4 bytes
	unsigned long  BPB_TotSec32; //4 bytes
//        union endboot  eb; //remaining part of bootsector
};


#define BYTE_PER_SEC	(U16)512

#define FAT12	12
#define FAT16	16
#define FAT32	32

//defines for special cluster values
//free cluster has value 0
//for fat32 don't use upper four bits ! ignore them
//cluster value of 0x10000000 is a FREE cluster in FAT32
 
//values for end of cluster chain
//ranges for example for FAT12 from 0xFF8 to 0xFFF
#define EOC12	0xFF8
#define EOC16	0xFFF8
#define EOC32	0x0FFFFFF8

//values for bad marked clusters
#define BADC12	0xFF7
#define BADC16	0xFFF7
#define BADC32	0x0FFFFFF7

//values for reserved clusters
//ranges for example for FAT12 from 0xFF0 to 0xFF6
#define RESC12	0xFF0
#define RESC16	0xFFF0
#define RESC32	0x0FFFFFF0

#ifdef USE_FAT32
 #define DISK_FULL 0xFFFFFFFF
#else
 #define DISK_FULL 0xFFFF
#endif

//File/Dir Attributes
#define ATTR_FILE	0x00 //not defined by MS ! I did it 
#define ATTR_READ_ONLY	0x01
#define ATTR_HIDDEN	0x02
#define ATTR_SYSTEM	0x04
#define ATTR_VOLUME_ID	0x08
#define ATTR_DIRECTORY	0x10
#define ATTR_ARCHIVE	0x20
#define ATTR_LONG_NAME  0x0F

//Char codes not allowed in a filename
//NOT checked yet
//0x22, 0x2A, 0x2B, 0x2C, 0x2E, 0x2F, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x5B, 0x5C, 0x5D, and 0x7C. 

struct DirEntry {
                 char DIR_Name[8];      //8 chars filename
                 char DIR_Ext[3];       //3 chars extension
                 unsigned char DIR_Attr;         //file attributes RSHA
                 unsigned char DIR_NTres;        //set to zero
                 unsigned char DIR_CrtTimeTenth; //creation time part in milliseconds
                 U16  DIR_CrtTime;      //creation time
                 U16  DIR_CrtDate;      //creation date
                 U16  DIR_LastAccDate;  //last access date (no time for this !)
                 U16  DIR_FstClusHI;  //first cluster high word                 
                 U16  DIR_WrtTime;      //last write time
                 U16  DIR_WrtDate;      //last write date
                 U16  DIR_FstClusLO;  //first cluster low word                 
                 unsigned long DIR_FileSize;     
                };


//Prototypes
extern unsigned char GetDriveInformation(void);
extern unsigned long GetFirstSectorOfCluster(U16 n);

extern unsigned char dirbuf[];   //buffer for directory entrys
extern U16 FileFirstCluster;
extern unsigned long FileSize;
extern unsigned long FirstRootSector;
extern unsigned char RootDirSectors;
extern unsigned long maxsect;           // last sector on drive
extern unsigned char secPerCluster;

#endif //FAT_H
